local addon = _G.PvPTimer
local L = addon.Lib.Locale
local LSM = addon.Lib.Media
local cdf = addon.Lib.CDFrames

local CopyTable = addon.CopyTable

-- generic getter/setter functions, use AceGUI's IDs to get/set values
local function getConfigValue(i)
	local db = addon.DB.profile
	
	for k = 1, #i do
		if not db then db = {} end
		db = db[i[k]]
	end
	
	return db
end

local function setConfigValue(i, v)
	local db = addon.DB.profile
	
	for k = 1, #i-1 do
		if not db[i[k]] then db[i[k]] = {} end
		db = db[i[k]]
	end
	
	db[i[#i]] = v
end

-- for colors
local function getColorValue(i)
	local t = getConfigValue(i)
	
	if t then
		return t.r, t.g, t.b, t.a
	else
		return 0, 0, 0, 1
	end
end

local function setColorValue(i, r, g, b, a)
	local t = {
		["r"] = r,
		["g"] = g,
		["b"] = b,
		["a"] = a,
	}
	setConfigValue(i, t)
end

-- for multiselect controls
local function getMultiValue(i, k)
	i[#i+1] = k
	return getConfigValue(i)
end

local function setMultiValue(i, k, v)
	i[#i+1] = k
	setConfigValue(i, v)
end

local const_bg = addon.Const.Battlegrounds
local const_arena = addon.Const.Arenas
local const_autohide = addon.Const.Autohide
local const_alerttarget = addon.Const.AlertTarget
local const_attach = addon.Const.Attach
local const_justify = addon.Const.Justify

-- option subtables
local alert = {
	CautionHeader = {
		order = 1,
		name = "CAUTION",
		type = "header",
	},
	CautionText = {
		order = 2,
		name = "Be careful when using these alerts, they can send out a lot of messages. People will likely yell at you or /ignore you if you spam alerts in a public channel.",
		type = "description",
		width = "full",
		fontSize = "medium",
	},
	EnabledHeader = {
		order = 10,
		name = "Enable Alert",
		type = "header",
	},
	Enabled = {
		order = 11,
		name = "Enable alerts",
		type = "toggle",
	},
	ZoneText = {
		order = 20,
		name = "Alert Target",
		type = "header",
	},
	BG = {
		order = 21,
		name = "Battlegrounds",
		type = "select",
		values = const_alerttarget,
	},
	BG_OnlyTarget = {
		order = 22,
		name = "Only target/focus",
		type = "toggle",
	},
	RatedBG = {
		order = 31,
		name = "Rated Battlegrounds",
		type = "select",
		values = const_alerttarget,
	},
	RatedBG_OnlyTarget = {
		order = 32,
		name = "Only target/focus",
		type = "toggle",
	},
	Arena = {
		order = 41,
		name = "Arenas",
		type = "select",
		values = const_alerttarget,
	},
	Arena_OnlyTarget = {
		order = 42,
		name = "Only target/focus",
		type = "toggle",
	},
	Else = {
		order = 98,
		name = "Everywhere else",
		type = "select",
		values = const_alerttarget,
	},
	Else_OnlyTarget = {
		order = 99,
		name = "Only target/focus",
		type = "toggle",
	},
}

local anchor = {
	Enabled = {
		order = 11,
		name = "Enable anchor",
		type = "toggle",
	},
	Locked = {
		order = 12,
		name = "Lock anchor",
		type = "toggle",
		set = function(i, v)
			cdf:UnlockAnchor(addon.Anchors[i[#i-1]], not v)
			setConfigValue(i, v)
		end,
		hidden = true,
	},
	Autohide = {
		order = 13,
		name = "If target is not a player",
		type = "select",
		-- style = "radio",
		values = const_autohide,
	},
	Width = {
		order = 21,
		name = "Width of anchor",
		type = "range",
		min = 100,
		max = 400,
		step = 5,
		width = "full",
	},
	Scale = {
		order = 22,
		name = "Scale",
		type = "range",
		min = 0.1,
		max = 5,
		step = 0.05,
		isPercent = true,
		width = "full",
	},
	Text_Enable = {
		order = 50,
		name = "Show anchor in",
		type = "header",
		width = "full",
	},
	BG = {
		order = 51,
		name = "Battlegrounds",
		type = "toggle",
	},
	RatedBG = {
		order = 52,
		name = "Rated Battlegrounds",
		type = "toggle",
	},
	Arena = {
		order = 53,
		name = "Arenas",
		type = "toggle",
	},
	Duel = {
		order = 54,
		name = "Duels",
		type = "toggle",
		hidden = true,
	},
	PvP = {
		order = 55,
		name = "PvP flagged",
		type = "toggle",
		hidden = true,
	},
	Else = {
		order = 59,
		name = "Everywhere else",
		type = "toggle",
	},
	Position = {
		order = 10,
		name = "Position",
		type = "group",
		args = {
			AttachPoint = {
				order = 11,
				name = "Attach point",
				type = "select",
				values = const_attach,
			},
			AttachFrame = {
				order = 12,
				name = "Attach to frame",
				type = "input",
				set = function(i, v)
					if not _G[v] or not _G[v].SetPoint then
						v = "UIParent"
						addon:Print("Invalid frame. Resetting to default.")
					end
					setConfigValue(i, v)
					addon:RefreshAnchors()
				end,
			},
			AttachRelative = {
				order = 13,
				name = "Frame point to attach",
				type = "select",
				values = const_attach,
			},
			AttachX = {
				order = 14,
				name = "Offset X",
				type = "range",
				softMin = -1000,
				softMax = 1000,
				min = -2000,
				max = 2000,
				step = 1,
				bigStep = 5,
				width = "full",
			},
			AttachY = {
				order = 15,
				name = "Offset Y",
				type = "range",
				softMin = -1000,
				softMax = 1000,
				min = -2000,
				max = 2000,
				step = 1,
				bigStep = 5,
				width = "full",
			},
		},
	},
	Header = {
		order = 20,
		name = "Header",
		type = "group",
		args = {
			CopySettings = {
				order = 1,
				name = "Copy settings",
				type = "execute",
				hidden = true,
			},
			PasteSettings = {
				order = 2,
				name = "Paste settings",
				type = "execute",
				disabled = function()
					if addon.CopyHeaderSettings == nil then
						return true
					else
						return false
					end
				end,
				hidden = true,
			},
			ShowClass = {
				order = 11,
				name = "Show class icon",
				type = "toggle",
			},
			ShowSpec = {
				order = 12,
				name = "Show spec icon",
				type = "toggle",
			},
			Color = {
				order = 13,
				name = "Header color",
				type = "color",
				hasAlpha = true,
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			ClassColor = {
				order = 14,
				name = "Use class color",
				type = "toggle",
			},
			Texture = {
				order = 15,
				name = "Header texture",
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				values = AceGUIWidgetLSMlists.statusbar,
			},
			Font = {
				order = 16,
				name = "Header font",
				type = "select",
				dialogControl = 'LSM30_Font',
				values = AceGUIWidgetLSMlists.font,
			},
			FontSize = {
				order = 17,
				name = "Header font size",
				type = "range",
				min = 1,
				max = 20,
				step = 1,
			},
			Justify = {
				order = 18,
				name = "Justify header text",
				type = "select",
				values = const_justify,
			},
		},
	},
	Bars = {
		order = 30,
		name = "Bars",
		type = "group",
		args = {
			CopySettings = {
				order = 1,
				name = "Copy settings",
				type = "execute",
				hidden = true,
			},
			PasteSettings = {
				order = 2,
				name = "Paste settings",
				type = "execute",
				disabled = function()
					if addon.CopyBarSettings == nil then
						return true
					else
						return false
					end
				end,
				hidden = true,
			},
			Color = {
				order = 11,
				name = "Bar color",
				type = "color",
				hasAlpha = true,
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			TypeColor = {
				order = 12,
				name = "Use spell category color",
				type = "toggle",
			},
			GrowUp = {
				order = 13,
				name = "Bars grow upwards",
				type = "toggle",
			},
			Columns = {
				order = 14,
				name = "Columns",
				type = "range",
				min = 1,
				max = 5,
				step = 1,
			},
			Texture = {
				order = 15,
				name = "Bar texture",
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				values = AceGUIWidgetLSMlists.statusbar,
			},
			Text_SpellName = {
				order = 20,
				name = "Spell Name",
				type = "header",
			},
			ShowName = {
				order = 21,
				name = "Show spell name",
				type = "toggle",
				width = "full",
			},
			NameFont = {
				order = 22,
				name = "Font",
				type = "select",
				dialogControl = 'LSM30_Font',
				values = AceGUIWidgetLSMlists.font,
				hidden = true,
			},
			NameFontSize = {
				order = 23,
				name = "Font size",
				type = "range",
				min = 1,
				max = 20,
				step = 1,
				hidden = true,
			},
			Text_Time = {
				order = 30,
				name = "Time Remaining",
				type = "header",
			},
			ShowTime = {
				order = 31,
				name = "Show time remaining",
				type = "toggle",
				width = "full",
			},
			TimeFont = {
				order = 32,
				name = "Font",
				type = "select",
				dialogControl = 'LSM30_Font',
				values = AceGUIWidgetLSMlists.font,
				hidden = true,
			},
			TimeFontSize = {
				order = 33,
				name = "Font size",
				type = "range",
				min = 1,
				max = 20,
				step = 1,
				hidden = true,
			},
		},
	},
}

local anchor_arena = addon:CopyTable(anchor)
anchor_arena.Autohide = nil
anchor_arena.Text_Enable = nil
anchor_arena.BG = nil
anchor_arena.RatedBG = nil
anchor_arena.Arena = nil
anchor_arena.Duel = nil
anchor_arena.PvP = nil
anchor_arena.Else = nil

anchor_arena.Position.args.NextX = {
	order = 18,
	name = "Distance between frames X",
	type = "range",
	softMin = -1000,
	softMax = 1000,
	min = -2000,
	max = 2000,
	step = 1,
	bigStep = 5,
	width = "full",
}
anchor_arena.Position.args.NextY = {
	order = 19,
	name = "Distance between frames Y",
	type = "range",
	softMin = -1000,
	softMax = 1000,
	min = -2000,
	max = 2000,
	step = 1,
	bigStep = 5,
	width = "full",
}

local spellcat = {
	offensive = {
		order = 101,
		type = "color",
		name = "Offensive CDs",
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	defensive = {
		order = 102,
		type = "color",
		name = "Defensive CDs",
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	interrupt = {
		order = 103,
		type = "color",
		name = "Interrupts and Silences",
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	cc = {
		order = 104,
		type = "color",
		name = "Crowd Control",
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	root = {
		order = 105,
		type = "color",
		name = "Roots and Snares",
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	misc = {
		order = 106,
		type = "color",
		name = "Miscellaneous Spells",
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
}


--main option table
addon.Options = {
	type = "group",
	name = "PvPTimer",
	get = getConfigValue,
	set = setConfigValue,
	args = {
		Settings = {
			order = 1,
			name = "Settings",
			type = "group",
			args = {
				EnableAddon = {
					name = "Enable addon",
					type = "toggle",
					set = function(i, v)
						setConfigValue(i, v)
						addon:ZONE_CHANGED_NEW_AREA();
					end,
				},
				OpenSpellConfig = {
					name = "Spell Configuration",
					type = "execute",
					func = function() addon:ToggleSpellConfig() end,
				},
			},
		},
		Anchors = {
			order = 2,
			name = "Anchors",
			type = "group",
			set = function(i, v)
				setConfigValue(i, v)
				addon:RefreshAnchors()
			end,
			args = {
				Anchor_Target = {
					order = 2,
					name = "Target",
					type = "group",
					args = anchor,
				},
				Anchor_Focus = {
					order = 3,
					name = "Focus",
					type = "group",
					args = anchor,
				},
				Anchor_Arena = {
					order = 4,
					name = "Arena Targets",
					type = "group",
					args = anchor_arena,
				},
			},
		},
		Alert = {
			order = 3,
			name = "Alert",
			type = "group",
			args = {
				offensive = {
					order = 101,
					type = "group",
					name = "Offensive CDs",
					args = alert,
				},
				defensive = {
					order = 102,
					type = "group",
					name = "Defensive CDs",
					args = alert,
				},
				interrupt = {
					order = 103,
					type = "group",
					name = "Interrupts and Silences",
					args = alert,
				},
				cc = {
					order = 104,
					type = "group",
					name = "Crowd Control",
					args = alert,
				},
				root = {
					order = 105,
					type = "group",
					name = "Roots and Snares",
					args = alert,
				},
				misc = {
					order = 106,
					type = "group",
					name = "Miscellaneous Spells",
					args = alert,
				},
				spec = {
					order = 999,
					type = "group",
					name = "Spec Detection",
					args = alert,
				},
			},
		},
		SpellCategories = {
			order = 4,
			name = "Spell Categories",
			type = "group",
			args = spellcat,
		},
	},
}
